/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/sync.ts > TAP > more custom internal imports > #failing-conditional => null > must match snapshot 1`] = `
ERR_PACKAGE_IMPORT_NOT_DEFINED
`

exports[`test/sync.ts > TAP > more custom internal imports > #failing-starmatch => null > must match snapshot 1`] = `
ERR_PACKAGE_IMPORT_NOT_DEFINED
`

exports[`test/sync.ts > TAP > more custom internal imports > #invalid => null > must match snapshot 1`] = `
ERR_MODULE_NOT_FOUND
`

exports[`test/sync.ts > TAP > more custom internal imports > #invalid-star-expand => null > must match snapshot 1`] = `
ERR_MODULE_NOT_FOUND
`

exports[`test/sync.ts > TAP > more custom internal imports > #not-found => null > must match snapshot 1`] = `
ERR_PACKAGE_IMPORT_NOT_DEFINED
`

exports[`test/sync.ts > TAP > more custom internal imports > #null => null > must match snapshot 1`] = `
ERR_PACKAGE_IMPORT_NOT_DEFINED
`

exports[`test/sync.ts > TAP > more custom internal imports > #starreplace => null > must match snapshot 1`] = `
ERR_PACKAGE_IMPORT_NOT_DEFINED
`

exports[`test/sync.ts > TAP > more custom internal imports > => null > must match snapshot 1`] = `
ERR_MODULE_NOT_FOUND
`
