/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/conditions.ts > TAP > resolve import types > must match snapshot 1`] = `
file://{CWD}/dist/esm/index.d.ts
`

exports[`test/conditions.ts > TAP > resolve require > must match snapshot 1`] = `
file://{CWD}/dist/commonjs/index.min.js
`

exports[`test/conditions.ts > TAP > resolve require types > must match snapshot 1`] = `
file://{CWD}/dist/commonjs/index.d.ts
`
