// import "x/blah/x/y/z"
// import "x/deep/a/b/c/d/y/x.js"
// import "x/deep/a/b/c/d/y/y.js"
// import "#deep/a/b/c/d/y"
import { glob } from '#x'
import * as yaml from '#y/'
import * as yamlUtils from '#u/ti/l'
console.error({ glob, yaml, yamlUtils })
